package cn.com.poc.expose.rest.impl;

import cn.com.poc.common.service.BosConfigService;
import cn.com.poc.common.utils.StringUtils;
import cn.com.poc.common.utils.UUIDTool;
import cn.com.poc.expose.dto.MarkdownToWordDto;
import cn.com.poc.expose.rest.Markdown2WordRest;
import cn.com.yict.framemax.core.exception.BusinessException;
import com.vladsch.flexmark.html.HtmlRenderer;
import com.vladsch.flexmark.parser.Parser;
import com.vladsch.flexmark.util.ast.Document;
import org.apache.poi.poifs.filesystem.DirectoryEntry;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.apache.poi.xwpf.usermodel.XWPFParagraph;
import org.apache.poi.xwpf.usermodel.XWPFRun;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.*;
import java.util.UUID;

/**
 * @author alex.yao
 * @date 2025/3/7
 */
@Component
public class Markdown2WordRestImpl implements Markdown2WordRest {

    @Resource
    private BosConfigService bosConfigService;


    @Override
    public String markdown2Word(MarkdownToWordDto dto) throws IOException {
        String markdown = dto.getMarkdown();
        if (StringUtils.isBlank(markdown)) {
            throw new BusinessException("markdown is blank");
        }
        String htmlContent = convertMarkdownToHtml(markdown);
        return convertHtmlToWord(htmlContent);
    }

    private String convertMarkdownToHtml(String markdown) {
        Parser parser = Parser.builder().build();
        HtmlRenderer renderer = HtmlRenderer.builder().build();
        Document document = parser.parse(markdown);
        return renderer.render(document);
    }

    private String convertHtmlToWord(String html) throws IOException {
        File file = File.createTempFile(UUIDTool.getUUID(), ".docx");
        XWPFDocument document = new XWPFDocument();
        XWPFParagraph paragraph = document.createParagraph();
        XWPFRun run = paragraph.createRun();
        run.setText(html);
        FileOutputStream outputStream = new FileOutputStream(file);
        ByteArrayInputStream bais = new ByteArrayInputStream(html.getBytes());//将字节数组包装到流中
        POIFSFileSystem poifs = new POIFSFileSystem();
        DirectoryEntry directory = poifs.getRoot();
        directory.createDocument("WordDocument", bais);
        poifs.writeFilesystem(outputStream);
        FileInputStream fileInputStream = new FileInputStream(file);
        return bosConfigService.upload(fileInputStream, "docx", "application/msword");
    }
}
